package audio;

public class Additive1 {

    public static class WaveDataAdd extends WaveData {

        public void harmonics(int nSamples, double normalizedAmplitude, double frequency) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1)
                normalizedAmplitude = 1;
            short amplitude = (short) (32767 * Math.abs(normalizedAmplitude) / 2); // metà dell'ampiezza massima

            double nSamplesPerCycle = 44100 / frequency;
            for (int i = 0; i < nSamples; i++) {
                double a1 = amplitude * Math.sin(2 * Math.PI * (i / nSamplesPerCycle));
                double a2 = amplitude / 2 * Math.sin(2 * Math.PI * 2 * (i / nSamplesPerCycle));
                double a3 = amplitude / 3 * Math.sin(2 * Math.PI * 3 * (i / nSamplesPerCycle));
                double a4 = amplitude / 5 * Math.sin(2 * Math.PI * 4 * (i / nSamplesPerCycle));
                double a5 = amplitude / 10 * Math.sin(2 * Math.PI * 5 * (i / nSamplesPerCycle));
                Sample s = new Sample((short)(a1 + a2 + a3 + a4 + a5), (short)(a1 + a2 + a3 + a4 + a5));
                audioSamples.add(s);
            }
        }

        public void sawtooth(int nSamples, double normalizedAmplitude, double frequency) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1)
                normalizedAmplitude = 1;
            short amplitude = (short) (32767 * Math.abs(normalizedAmplitude) / 20); // un ventesimo dell'ampiezza massima

            double nSamplesPerCycle = 44100 / frequency;
            for (int i = 0; i < nSamples; i++) {
                double a1 = 10 * amplitude * Math.sin(2 * Math.PI * (i / nSamplesPerCycle));
                double a2 = 5 * amplitude * Math.sin(2 * Math.PI * 2 * (i / nSamplesPerCycle));
                double a3 = 3.3 * amplitude * Math.sin(2 * Math.PI * 3 * (i / nSamplesPerCycle));
                double a4 = 2.5 * amplitude * Math.sin(2 * Math.PI * 4 * (i / nSamplesPerCycle));
                double a5 = 2 * amplitude * Math.sin(2 * Math.PI * 5 * (i / nSamplesPerCycle));
                double a6 = 1.6 * amplitude * Math.sin(2 * Math.PI * 6 * (i / nSamplesPerCycle));
                double a7 = 1.4 * amplitude * Math.sin(2 * Math.PI * 7 * (i / nSamplesPerCycle));
                double a8 = 1.25 * amplitude * Math.sin(2 * Math.PI * 8 * (i / nSamplesPerCycle));
                double a9 = 1.1 * amplitude * Math.sin(2 * Math.PI * 9 * (i / nSamplesPerCycle));
                double a10 = amplitude * Math.sin(2 * Math.PI * 10 * (i / nSamplesPerCycle));
                Sample s = new Sample((short)(a1 + a2 + a3 + a4 + a5 + a6 + a7 + a8 + a9 + a10), (short)(a1 + a2 + a3 + a4 + a5 + a6 + a7 + a8 + a9 + a10));
                audioSamples.add(s);
            }
        }

        public void sin2(int nSamples, double normalizedAmplitude, double frequency1, double frequency2) {
            audioSamples.clear();

            if (Math.abs(normalizedAmplitude) > 1)
                normalizedAmplitude = 1;
            short amplitude = (short) (32767 * Math.abs(normalizedAmplitude) / 2); // metà dell'ampiezza massima

            double nSamplesPerCycle1 = 44100 / frequency1;
            double nSamplesPerCycle2 = 44100 / frequency2;
            for (int i = 0; i < nSamples; i++) {
                double a1 = amplitude * Math.sin(2 * Math.PI * (i / nSamplesPerCycle1));
                double a2 = amplitude * Math.sin(2 * Math.PI * (i / nSamplesPerCycle2));
                Sample s = new Sample((short)(a1 + a2), (short)(a1 + a2));
                audioSamples.add(s);
            }
        }
    }
    
    public static void main(String[] args) {
        WaveDataAdd wd = new WaveDataAdd();
        wd.harmonics(44100, 1, 440);
        wd.saveSamplesToDisk("harmonics.wav");

        wd = new WaveDataAdd();
        wd.sawtooth(44100, 1, 440);
        wd.saveSamplesToDisk("sawtooth.wav");

        wd = new WaveDataAdd();
        wd.sin2(88200, 1, 440, 450);
        wd.saveSamplesToDisk("sin2.wav");
    }
}
